# Generated by roxygen2: do not edit by hand

S3method(summary,aftable)
S3method(tbl_sum,aftable)
export(as_aftable)
export(at_template_aftable)
export(at_template_workflow)
export(create_aftable)
export(create_config_yaml)
export(generate_workbook)
export(is_aftable)
import(openxlsx2)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,mutate)
importFrom(pillar,tbl_sum)
importFrom(purrr,compact)
importFrom(purrr,list_modify)
importFrom(purrr,map_depth)
importFrom(purrr,pluck)
importFrom(purrr,pluck_exists)
importFrom(purrr,pwalk)
importFrom(rlang,'%||%')
importFrom(rlang,.data)
importFrom(stringr,str_extract)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(tibble,tibble)
importFrom(tidyselect,where)
