## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#"
)

## ----results='asis', echo = FALSE---------------------------------------------
cat("```xml\n")
cat(readLines(system.file("ext-data",
                          "config.yaml",
                          package = "aftables")),
    sep = "\n")
cat("\n```\n")

## ----config example 1, eval = FALSE-------------------------------------------
# my_wb <- generate_workbook(aftable = demo_aftable)

## ----config example 2, eval = FALSE-------------------------------------------
# my_wb <- generate_workbook(
#   aftable = demo_aftable,
#   config_name = "workbook1"
# )

## ----config example 3, eval = FALSE-------------------------------------------
# dir.create("configs")
# 
# create_config_yaml(
#   path = "configs",
#   open_config = FALSE
# )
# 
# my_wb <- generate_workbook(
#   aftable = demo_aftable,
#   config_path = "configs/config.yaml"
# )

## ----config example 4, eval = FALSE-------------------------------------------
# dir.create("configs")
# 
# create_config_yaml(
#   path = "configs",
#   open_config = FALSE
# )
# 
# my_wb <- generate_workbook(
#   aftable = demo_aftable,
#   config_path = "configs/config.yaml",
#   config_name = "workbook1"
# )

