#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import openxlsx2
#' @importFrom dplyr across
#' @importFrom dplyr mutate
#' @importFrom dplyr everything
#' @importFrom dplyr filter
#' @importFrom dplyr all_of
#' @importFrom tidyselect where
#' @importFrom rlang .data
#' @importFrom rlang '%||%'
#' @importFrom pillar tbl_sum
#' @importFrom purrr pwalk
#' @importFrom purrr pluck
#' @importFrom purrr pluck_exists
#' @importFrom purrr list_modify
#' @importFrom purrr map_depth
#' @importFrom purrr compact
#' @importFrom stringr str_extract
#' @importFrom stringr str_replace
#' @importFrom stringr str_replace_all
#' @importFrom tibble tibble
## usethis namespace: end
NULL
