% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aftable.R
\name{tbl_sum.aftable}
\alias{tbl_sum.aftable}
\title{Provide A Succinct Summary Of An 'aftable' Object}
\usage{
\method{tbl_sum}{aftable}(x, ...)
}
\arguments{
\item{x}{An aftable-class object to summarise.}

\item{...}{Other arguments to pass.}
}
\value{
Named character vector.
}
\description{
A brief text description of an aftable-class object.
}
\examples{
# Print with description
print(demo_aftable)

# Print description only (package 'tibble' must be installed)
tibble::tbl_sum(demo_aftable)

}
