% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addin.R
\name{at_template_workflow}
\alias{at_template_workflow}
\title{Insert full 'aftables' template workflow}
\usage{
at_template_workflow()
}
\value{
Empty list. Function is used for side effect.
}
\description{
Insert at the cursor (a) demo templates for cover, contents and notes
tables, and (b) a call to \code{\link{create_aftable}} pre-filled with
demo data.
}
