% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{demo_df}
\alias{demo_df}
\title{A Demo 'data.frame' Object}
\format{
A data.frame with 6 rows and 7 columns:
\describe{
\item{tab_title}{Character. Text to appear on each sheet's tab.}
\item{sheet_type}{Character. The content type for each sheet: 'cover', 'contents', 'notes', or 'tables'.}
\item{sheet_title}{Character. The title that will appear in cell A1 (top-left) of each sheet.}
\item{blank_cells}{Character. An explanation for any blank cells in the table.}
\item{custom_rows}{List-column of character vectors. Additional arbitrary pre-table information provided by the user.}
\item{source}{Character. The origin of the data, if relevant.}
\item{table}{List-column of data.frames (apart from the cover, which is a list) containing the statistical tables.}
}
}
\usage{
demo_df
}
\description{
A pre-created data.frame ready to be converted to an aftables-class object
with \code{\link{as_aftable}} and then an 'openxlsx2' wbWorkbook-class object
with \code{\link{generate_workbook}}.
}
\keyword{datasets}
