% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addin.R
\name{at_template_aftable}
\alias{at_template_aftable}
\title{Insert 'create_aftable' template}
\usage{
at_template_aftable()
}
\value{
Empty list. Function is used for side effect.
}
\description{
Insert at the cursor a template for \code{\link{create_aftable}} from the
'aftables' package, pre-filled with demo data.
}
