% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{demo_workbook}
\alias{demo_workbook}
\title{A Demo 'Workbook' Object}
\format{
An 'openxlsx2' wbWorkbook-class object with 5 sheets.
}
\usage{
demo_workbook
}
\description{
A pre-created 'openxlsx2' wbWorkbook-class object generated from an
aftables-class object with \code{\link{generate_workbook}}.
}
\keyword{datasets}
