% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aftable.R
\name{as_aftable}
\alias{as_aftable}
\alias{is_aftable}
\title{Coerce To An 'aftable' Object}
\usage{
as_aftable(x)

is_aftable(x)
}
\arguments{
\item{x}{A data.frame object to coerce.}
}
\value{
\code{as_aftable} returns an object of class aftable if possible.
\code{is_aftable} returns \code{TRUE} if the object has class
aftable, otherwise \code{FALSE}.
}
\description{
Functions to check if an object is an aftable, or coerce it if possible.
}
\examples{
as_aftable(demo_df)
is_aftable(demo_aftable)

}
