% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aftables-package.R
\docType{package}
\name{aftables-package}
\alias{aftables}
\alias{aftables-package}
\title{aftables: Create Spreadsheet Publications Following Best Practice}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Generate spreadsheet publications that follow best practice guidance from the UK government's Analysis Function, available at \url{https://analysisfunction.civilservice.gov.uk/policy-store/releasing-statistics-in-spreadsheets/}, with a focus on accessibility. See also the 'Python' package 'gptables'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://best-practice-and-impact.github.io/aftables/}
  \item \url{https://github.com/best-practice-and-impact/aftables}
  \item Report bugs at \url{https://github.com/best-practice-and-impact/aftables/issues}
}

}
\author{
\strong{Maintainer}: Olivia Box Power \email{Olivia.BoxPower@dhsc.gov.uk} [contributor]

Authors:
\itemize{
  \item Matt Dray \email{mwdray@gmail.com}
}

Other contributors:
\itemize{
  \item Tim Taylor [contributor]
  \item Matt Kerlogue [contributor]
  \item Crown Copyright [copyright holder]
  \item Zachary Gleisner \email{Zachary.Gleisner@dhsc.gov.uk} [contributor]
}

}
\keyword{internal}
